#ifndef DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES_DX_MARKETMESSAGE_INC
#define DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES_DX_MARKETMESSAGE_INC

#include <string>
#include <stdint.h>

#include "dxapi/dxapi.h"
#include "dfp/DecimalNative.hpp"
#include "src/codecs/NativeMessage.h"

using namespace std;
using namespace DxApi;

using namespace deltix::dfp;

namespace dx_BITTREX::deltix::timebase::api::messages {

class Dx_MarketMessage : public NativeMessage {
private:
    std::string __class_type__ = "deltix.timebase.api.messages.MarketMessage";
    std::string __class_guid__ = "64406423fb2b4ajpjsoe_1d";

public:
    virtual std::string & getTypeName() { return __class_type__; }
    virtual std::string & getGuid() { return __class_guid__; }

    virtual void decode(DataReader &reader) {
        currencyCode_ = reader.readInt16();
        originalTimestamp_ = reader.readTimestamp();
        sequenceNumber_ = reader.readInt64();
        has_sourceId_ = reader.readAlphanumeric(sourceId_, 10);
    }

    virtual void encode(DataWriter &writer) {
        writer.writeInt16(currencyCode_);
        writer.writeTimestamp(originalTimestamp_);
        writer.writeInt64(sequenceNumber_);
        has_sourceId_ ? writer.writeAlphanumeric(10, sourceId_) :  writer.writeAlphanumericNull(10);
    }

    inline void clear() {
        currencyCode_ = DxApi::INT16_NULL;
        originalTimestamp_ = DxApi::TIMESTAMP_NULL;
        sequenceNumber_ = DxApi::INT64_NULL;
        sourceId_.clear();
        has_sourceId_ = false;
    }

    inline void setCurrencyCode(int16_t value) {
        this->currencyCode_ = value;
    }

    inline void setOriginalTimestamp(int64_t value) {
        this->originalTimestamp_ = value;
    }

    inline void setSequenceNumber(int64_t value) {
        this->sequenceNumber_ = value;
    }

    inline void setSourceId(const string &value) {
        this->sourceId_ = value;
        has_sourceId_ = true;
    }

    virtual string toString() {
        stringstream ss;
        ss << NativeMessage::toString() << ", ";
        ss << "  currencyCode_: "; 
        if (currencyCode_ == DxApi::INT16_NULL) { ss << "[NULL]"; } else { ss << currencyCode_; }
        ss << ", ";
        ss << "  originalTimestamp_: "; 
        if (originalTimestamp_ == DxApi::TIMESTAMP_NULL) { ss << "[NULL]"; } else { ss << originalTimestamp_; }
        ss << ", ";
        ss << "  sequenceNumber_: "; 
        if (sequenceNumber_ == DxApi::INT64_NULL) { ss << "[NULL]"; } else { ss << sequenceNumber_; }
        ss << ", ";
        ss << "  sourceId_: "; 
        if (!has_sourceId_) { ss << "[NULL]"; } else { ss << sourceId_; }
        return ss.str();
    };

    int16_t currencyCode_;
    int64_t originalTimestamp_;
    int64_t sequenceNumber_;
    string sourceId_;

    bool has_sourceId_ = false;
};

} // namespace dx_BITTREX::deltix::timebase::api::messages

#endif /* DX_BITTREX__DELTIX__TIMEBASE__API__MESSAGES_DX_MARKETMESSAGE_INC */
