package deltix.samples.test;

import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import deltix.qsrv.hf.tickdb.pub.*;
import deltix.timebase.api.messages.securities.Index;
import deltix.timebase.api.messages.securities.Equity;
import deltix.timebase.api.messages.securities.Future;
import deltix.timebase.api.messages.securities.ETF;
import deltix.util.lang.Util;

/**
 * Creates securities stream test
 *
 * @author alex
 */
public class Test_DF1 extends TDBTestBase {

    @Test
    public void testSecurityStream1() {
        DXTickDB client = getTickDb();
        String name = "securities";
        deleteIfExists(name);

        DXTickStream stream = client.createStream (name, name, name, 1);
        StreamConfigurationHelper.setMeta(stream);

        try (TickLoader loader = stream.createLoader()){
            Index index = new Index();
            index.setSymbol("ETF");
            loader.send(index);
        }

        TickCursor cursor = stream.select(0, null);
        try {
            assertTrue(cursor.next());
            assertEquals(1, stream.listEntities().length);
        } finally {
            Util.close(cursor);
        }
    }

    public void testWrite() {
        DXTickDB db = getTickDb();
        String name = "sss";
        deleteIfExists(name);

        DXTickStream stream = db.createStream (name, name, name, 1);
        StreamConfigurationHelper.setMeta(stream);

        TickLoader loader = null;
        try {
            LoadingOptions loadingOptions = new LoadingOptions();
            loader = stream.createLoader(loadingOptions);

            Index index = new Index();
            index.setSymbol("XXL");
            index.setTimeStampMs(System.currentTimeMillis());
            loader.send(index);

        } finally {
            Util.close(loader);
        }

        db.close();
        db.open(false);

        stream = db.getStream (name);

        try {
            LoadingOptions loadingOptions = new LoadingOptions();
            loader = stream.createLoader(loadingOptions);

            Equity index = new Equity();
            index.setSymbol("XXX");
            index.setTimeStampMs(System.currentTimeMillis());
            loader.send(index);

        } finally {
            Util.close(loader);
        }
    }

    @Test
    public void testSecurityStream() {
        DXTickDB client = getTickDb();
        String name = "securities_test";
        deleteIfExists(name);

        DXTickStream stream = client.createStream (name, name, name, 1);
        StreamConfigurationHelper.setMeta(stream);

        TickLoader loader = null;
        try {
            loader = stream.createLoader();
            Equity equity = new Equity();
            equity.setSymbol("MSFT");
            loader.send(equity);

            Future future = new Future();
            future.setSymbol("MG");
            loader.send(future);

            ETF etf = new ETF();
            etf.setSymbol("ETF");
            loader.send(etf);

            loader.close();
        } finally {
            Util.close(loader);
        }
       
        SelectionOptions options = new SelectionOptions();
        options.reversed = true;
        TickCursor cursor = stream.select(Long.MIN_VALUE, options);
        cursor.next();

        stream.clear();

        try {
            loader = stream.createLoader();
            Equity equity = new Equity();
            equity.setSymbol("IBM");
            loader.send(equity);

            ETF etf = new ETF();
            etf.setSymbol("ETF");
            loader.send(etf);

            loader.close();
            loader = null;
        } finally {
            Util.close(loader);
        }

        stream.clear();

        try {
            loader = stream.createLoader();
            Equity equity = new Equity();
            equity.setSymbol("IBM");
            loader.send(equity);

            ETF etf = new ETF();
            etf.setSymbol("ETF");
            loader.send(etf);

            loader.close();
            loader = null;
        } finally {
            Util.close(loader);
            Util.close(cursor);
        }
    }
    
    
}
