import sys, os, struct

# Note: install dxapi from pip nexus repository 
#       pip install --extra-index-url https://nexus.deltixhub.com/repository/epm-rtc-public-python/simple
# or local directory from QuantServer installation:
#       pip install /path/to/QuantServer/python
#    or pip install .. (from current directory)

# The sample works with dxapi v5.4.28+
# In the sample we will create new stream from generated schema.

import dxapi

def main():
    # Timebase URL specification, pattern is "dxtick://<host>:<port>"
    timebase = 'dxtick://localhost:8011'
    try:
        # Create timebase connection
        db = dxapi.TickDb.createFromUrl(timebase)

        # Open in write mode
        db.open(False)

        print('Connected to ' + timebase)

        stream_key = 'trade_bbo_generated'

        # Delete stream if it already exists to create new
        stream = db.getStream(stream_key)
        if stream != None:
            stream.deleteStream()
        
        # Generate schema of new stream
        schema = db.generateSchema(['deltix.timebase.api.messages.TradeMessage', 
            'deltix.timebase.api.messages.BestBidOfferMessage'])

        # Create new stream using generateSchema
        options = dxapi.StreamOptions()
        options.schema(schema)
        stream = db.createStream(stream_key, options)
        
        writeSomeData(stream)
    finally:  # database connection should be closed anyway
        if (db.isOpen()):
            db.close()
        print("Connection " + timebase + " closed.")

def writeSomeData(stream):
    with stream.tryLoader(dxapi.LoadingOptions()) as loader:
        tradeMessage = dxapi.InstrumentMessage()
        tradeMessage.typeName = 'deltix.timebase.api.messages.TradeMessage'
        bboMessage = dxapi.InstrumentMessage()
        bboMessage.typeName = 'deltix.timebase.api.messages.BestBidOfferMessage'
        for i in range(10):
            tradeMessage.instrumentType = 'EQUITY'
            tradeMessage.symbol = 'AAPL' if i % 2 == 0 else 'MSFT'
            tradeMessage.price = i * 1.1
            tradeMessage.size = i * 2.2
            tradeMessage.aggressorSide = 'SELL' if i % 2 == 0 else 'BUY'
            loader.send(tradeMessage)
            
            bboMessage.instrumentType = 'EQUITY'
            bboMessage.symbol = 'AAPL' if i % 2 == 0 else 'MSFT'
            bboMessage.bidPrice = i * 1.1
            bboMessage.bidSize = i * 2.2
            bboMessage.offerPrice = i * 3.3
            bboMessage.offerSize = i * 4.4
            loader.send(bboMessage)


if __name__ == '__main__':
    main()