import sys, os, inspect, struct
import math
import time
import calendar
from datetime import datetime
import threading 

# Note: install dxapi from pip nexus repository 
#       pip install --extra-index-url https://nexus.deltixhub.com/repository/epm-rtc-public-python/simple
# or local directory from QuantServer installation:
#       pip install /path/to/QuantServer/python
#    or pip install .. (from current directory)

import dxapi

# Timebase URL specification, pattern is "dxtick://<host>:<port>"
timebase = 'dxtick://localhost:8011'

def readCursor(threadName, streamKey, startTime):
    try:
        # Create timebase connection
        db = dxapi.TickDb.createFromUrl(timebase)
        
        # Open in read-only mode
        db.open(False)
        
        print('Connected to ' + timebase)

        # Get stream from the timebase
        stream = db.getStream(streamKey)
        if stream == None:
            raise Exception('Stream ' + streamKey + ' not found, please, create stream')
        
        options = dxapi.SelectionOptions()
        options.live = True
        # Create cursor using defined message types and entities
        cursor = stream.select(startTime, options, None, None)
        ii = 0
        try:
            while True:
                state = cursor.nextIfAvailable()
                if state == dxapi.OK:
                    message = cursor.getMessage()
                    ii += 1
                    if ii % 10 == 0:
                        print("Read " + str(ii) + " messages from " + streamKey)
                        print(str(message))
                elif state == dxapi.END_OF_CURSOR:
                    print("END OF CURSOR")
                    break
        finally:
            # cursor should be closed anyway
            cursor.close()
            cursor = None
    finally:
        # database connection should be closed anyway
        if (db.isOpen()):
            db.close()
            print("Connection " + timebase + " closed.")
        
        
t1 = threading.Thread(target = readCursor, args = ("Thread-1", "trade_bbo", 0, ) )
t2 = threading.Thread(target = readCursor, args = ("Thread-2", "universal", 0, ) )

t1.start()
t2.start()

t1.join()
t2.join()
