import sys, os, struct

# Note: install dxapi from pip nexus repository 
#       pip install --extra-index-url https://nexus.deltixhub.com/repository/epm-rtc-public-python/simple
# or local directory from QuantServer installation:
#       pip install /path/to/QuantServer/python
#    or pip install .. (from current directory)

import dxapi
from pprint import pprint

def main():
    try:
        db = dxapi.TickDb.createFromUrl("dxtick://localhost:8011")
        db.open(True)
        stream = db.getStream("trade_bbo")
        if stream == None:
            raise Exception('Stream trade_bbo not found, please, create stream')
            
        messages = df_from_stream(db, stream, ['timestamp', 'symbol'])
        pprint(messages.head())
    finally:
        if (db.isOpen()):
            db.close()

def df_from_stream(db, stream, fields=None, ts_from=0, ts_to=922337203685477580):
    if 'pandas' not in sys.modules:
        import pandas as pd
    else:
        pd = sys.modules['pandas']
    table = dxapi.stream_to_dict(db, stream, fields, ts_from, ts_to)
    table = pd.DataFrame(table)
    table.timestamp = pd.to_datetime(table.timestamp)
    return table
    
if __name__ == '__main__':
    main()