import sys, os, struct

# Note: install dxapi from pip nexus repository 
#       pip install --extra-index-url https://nexus.deltixhub.com/repository/epm-rtc-public-python/simple
# or local directory from QuantServer installation:
#       pip install /path/to/QuantServer/python
#    or pip install .. (from current directory)

# The sample works with dxapi v5.4.28+
# In the sample we will update stream schema.

import dxapi

def main():
    # Timebase URL specification, pattern is "dxtick://<host>:<port>"
    timebase = 'dxtick://localhost:8011'
    try:
        # Create timebase connection
        db = dxapi.TickDb.createFromUrl(timebase)

        # Open in write mode
        db.open(False)

        print('Connected to ' + timebase)

        stream = getOrCreateStream(db, 'trade_bbo_generated')
            
        # Let's change stream schema and add new field to TradeMessage
        schema = stream.metadata()
        tradeType = schema.findType('TradeMessage')
        tradeType.fields.append(
            dxapi.FieldDef('newField', 'New Field', 'New Field', dxapi.DataTypeDef('VARCHAR', 'ALPHANUMERIC(10)', False))
        )
        
        # New field is not nullable, so we should set default value mapping for it
        defaults = { 'deltix.timebase.api.messages.TradeMessage:newField' : 'SOMEVALUE' }
        
        # Apply new schema to stream
        stream.changeSchema(schema=schema, defaults=defaults)

        readMessages(stream)
    finally:  # database connection should be closed anyway
        if (db.isOpen()):
            db.close()
        print("Connection " + timebase + " closed.")

def getOrCreateStream(db, stream_key):
    stream = db.getStream(stream_key)
    if stream != None:
        return stream

    # Generate schema of new stream
    schema = db.generateSchema(['deltix.timebase.api.messages.TradeMessage',
                                'deltix.timebase.api.messages.BestBidOfferMessage'])

    # Create new stream using generateSchema
    options = dxapi.StreamOptions()
    options.schema(schema)
    return db.createStream(stream_key, options)

def readMessages(stream):
    with stream.trySelect(0, dxapi.SelectionOptions(), None, None) as cursor:
        while cursor.next():
            print(cursor.getMessage())

if __name__ == '__main__':
    main()