# Note: install dxapi from pip nexus repository 
#       pip install --extra-index-url https://nexus.deltixhub.com/repository/epm-rtc-public-python/simple
# or local directory from QuantServer installation:
#       pip install /path/to/QuantServer/python
#    or pip install .. (from current directory)

# Note: the sample is valid for dxapi 5.5.3 and later

import dxapi
from dxapi import pandas_utils

def main():
    try:
        db = dxapi.TickDb.createFromUrl("dxtick://localhost:8011")
        db.open(False)
        
        fields=['open', 'close', 'high', 'low']
        symbols=['IBM', 'AAPL'] # Use None for all symbols
        types=None # Use None for all types
        df = pandas_utils.read_frame(db,
                                     streams=['Daily'],
                                     fields=fields,
                                     tickers=symbols,
                                     types=types,
                                     from_time_str='2001-01-01',
                                     to_time_str='2001-05-02')
        print(df)
        
        # Write DataFrame to new stream
        binding = pandas_utils.bind_frame(db, 'Daily_new', df)
        pandas_utils.write_frame(db, binding, df)
    finally:
        if (db.isOpen()):
            db.close()

if __name__ == '__main__':
    main()